/*===========================================================================
 * Project         : Mandrake Linux
 * Module          : rpmstats
 * File            : rpmstats.c
 * Version         : $Id: rpmstats.c 264547 2010-01-05 08:48:36Z pterjan $
 * Author          : Frederic Lepied
 * Created On      : Sat Feb 17 16:08:53 2001
 *===========================================================================*/

static const char rcs_id[] = "$Id: rpmstats.c 264547 2010-01-05 08:48:36Z pterjan $";
static const char compile_id[] = "$Compile: " __FILE__ " " __DATE__ " " __TIME__ " $";

/*
 **********************************************************************
 *
 * Included files
 *
 **********************************************************************
 */

#include <rpm/rpmlib.h>
#include <stdio.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <sys/time.h>
#include <regex.h>
#include <string.h>

#ifdef RPM_42
#include <rpm/rpmdb.h>
#include <rpm/rpmts.h>
#endif

int
main(int argc, char * argv[])
{
  rpmdbMatchIterator mi;
#ifdef RPM_42
  rpmts ts;
#else
  rpmdb db = NULL;
#endif
  Header h;
  int data_size ;
  char ** file_list = NULL;
  struct timeval tv;
  regex_t rx;
  
  gettimeofday(&tv, NULL);

#ifdef DEBUG
  printf("%d\n", tv.tv_sec);
#endif
  
  rpmReadConfigFiles(NULL, NULL);

#ifdef RPM_42
  ts = rpmtsCreate();
  rpmtsSetRootDir(ts, "");
  if (rpmtsOpenDB(ts, O_RDONLY)) {
    fprintf(stderr, "unable to open rpm database\n");
    exit(1);
  }
#else
  if (rpmdbOpen("", &db, O_RDONLY, 0644)) {
    fprintf(stderr, "unable to open rpm database\n");
    exit(1);
  }
#endif

#ifdef RPM_42
  mi = rpmtsInitIterator(ts, RPMDBI_PACKAGES, NULL, 0);
#else
  mi = rpmdbInitIterator(db, RPMDBI_PACKAGES, NULL, 0);
#endif
  
  if (regcomp(&rx, "/man/man\\|/usr/share/icons\\|/usr/lib/menu\\|/etc/menu-methods", 0)) {
    perror("regcomp");
    exit(1);
  }
  
  while ((h = rpmdbNextIterator(mi)) != NULL) {
    char *np, *vp, *rp;
    int type, count, loop;
    struct stat buf;
    int max = 0;
    const char *file = NULL;
    const char *fname = NULL;
    
    if (headerGetEntry(h, RPMTAG_NAME, &type, (void **) &np, &count) &&
	headerGetEntry(h, RPMTAG_VERSION, &type, (void **) &vp, &count) &&
	headerGetEntry(h, RPMTAG_RELEASE, &type, (void **) &rp, &count)) {

#ifdef RPM_42
      rpmfi fi;
      fi = rpmfiNew(NULL, h, RPMTAG_BASENAMES, 0);
      fi = rpmfiInit(fi, 0);
      while (rpmfiNext(fi) >= 0) {
        fname = rpmfiFN(fi);
#else
      rpmBuildFileList(h, (const char ***)&file_list, &data_size);
      
      for(loop=0; loop<data_size; loop++) {
	fname = file_list[loop];
#endif
	if (regexec(&rx, fname, 0, NULL, 0)) {
	  if (stat(fname, &buf) == 0) {
	    buf.st_atime = (buf.st_atime > buf.st_mtime) ? buf.st_atime : buf.st_mtime;
#ifdef DEBUG
	    fprintf(stderr, "%s %d\n", fname, buf.st_atime);
#endif
	    if (S_ISREG(buf.st_mode) && max < buf.st_atime) {
	      max = buf.st_atime;
	      file = fname;
	    }
	  } else {
	    perror(fname);
	  }
	} else {
#ifdef DEBUG
fprintf(stderr, "file %s match regex\n", fname);
#endif
	}
      }
      if (max != 0) {
	printf("%ld %s-%s-%s %s\n", (tv.tv_sec-max)/(3600*24), np, vp, rp, file);
      } else {
	printf("0 %s-%s-%s\n", np, vp, rp);
      }
#ifdef RPM_42
      fi = rpmfiFree(fi);
#endif
      free(file_list);
    }
  }

  rpmdbFreeIterator(mi);

  return 0;
}

/*
 * Local variables:
 * mode: c
 * compile-command: "gcc -Wall -g rpmstats.c -o rpmstats -lrpm -lrpmio -lpopt"
 * End:
 *
 * rpmstats.c ends here
 */
